import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.*;
import java.io.*;

/**
 * The XmlParser class is parsing objects to and from XML. 
 * Utilizing the KXML-parser, which is a lightweight xml parser developed for
 * use on mobile handsets. 
 * 
 * @author Hvard Holje
 * @since May 9, 2007
 */
public class XmlParser {

	private String root;
    private UserSession session;

    public XmlParser() {}
    
    /**
     * This method creates a AuthenticationRequest-object from the
     * xml data parsed from the inputStream
     * @param is
     * @return AuthenticationRequest
     */
    public UserSession getAuthenticationResponse(String input) {
        root = "AuthenticationResponse";    
    	
    	session = new UserSession();
    	
    	KXmlParser parser = new KXmlParser();
        
    	try {
    		
    		//Converts input string to stream data
    		byte[] xmlByteArray = input.getBytes();
			ByteArrayInputStream xmlStream = new 
			ByteArrayInputStream( xmlByteArray );
			InputStreamReader xmlReader = new InputStreamReader( xmlStream );
			
			parser.setInput(xmlReader);
			
			parser.nextTag();
	    	
			parser.require(XmlPullParser.START_TAG, null, root);
	    	
			while (parser.nextTag() != XmlPullParser.END_TAG) {
				readXMLData(parser);
			}
			
			parser.require(XmlPullParser.END_TAG, null, root);
			parser.next();	
			parser.require(XmlPullParser.END_DOCUMENT, null, null);		

	    } catch (Exception e) {
             e.printStackTrace ();
             System.out.println("Error in getAuthenticationResponse(): " + e);
             return null;
	    }
        return session;
    }
    
   /**
    * This method parses the xml data
    * @param parser
    * @throws IOException
    * @throws XmlPullParserException
    */
    private void readXMLData(KXmlParser parser) throws IOException, XmlPullParserException {
    	
    	//Parsing the XML String
    	parser.require(XmlPullParser.START_TAG, null, null);
		String name = parser.getName();
		String text = parser.nextText();
		
		if (name.equals("SecurityAssociation")) {
			session.setSecurityAssociation(text);
		}
		else if (name.equals("ErrorDescription")) {
			session.setErrorMsg(text);
		}
			
		parser.require(XmlPullParser.END_TAG, null, name);

    }
    
    /**
     * Creates the xml AuthenticationRequest with user PIN
     * @param authResponse
     * @return String xmlAuthResponse
     */
    public String createAuthenticationResponse(String userPin) {
        String xml = //"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" +
        			"<AuthenticationRequest>" +
        			"<Pin>"+userPin+"</Pin>" +
        			"</AuthenticationRequest>";    	
    	
    	return xml.trim();
    }

    
} //END Class XmlParser