import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.Enumeration;


/*
 * This is the Service Provider servlet.
 * It catches incoming service requests, checks the Security Association from the given userID and
 * create a session id if the security association is valid
 * If valid session id is provided the user is given access to the service immediately.
 *
 * @author Hvard Holje
 */
public class Service extends HttpServlet {

	// The response message to send to client
	String message;

	boolean userAuthenticated = false;

	/*
	 * Process the incoming request
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException{


		// Retrieve the session object corresponding to this request.
		HttpSession session = request.getSession();

		if (session.isNew()) {

			System.out.println("Session isNew");

			// Retrieve the Security Association which is passed to us as a header parameter.
			String SA = request.getHeader("SA");

			// Now look up the corresponding user information and check whether user
			// is authenticated against the IDP or not
			if(SA != null && SA != "") {
				userAuthenticated = lookupUser(SA);
			}



			if(userAuthenticated) {

				message = "New session created: " + session.getId();
			}

			else { //Redirect to local Supplicant
				session.invalidate();
				message = "Not authenticated, Security Association not valid..";

			    String redirectURL = "http://127.0.0.1:4035";
			    response.sendRedirect(redirectURL);
			}


			// Send the message to the client.
			response.setContentType("text/plain");
			response.setContentLength(message.length());
			PrintWriter out = response.getWriter();
			out.println(message);


		} else {
			System.out.println("Session is NOT new");

			// The client is providing a sessionID and is authorized


			// Return "account balance"
			message = "Account 1565 45 356: 45506,50 NOK\n";
			message += "Account 1565 45 357: 257600,00 NOK\n";

			// Send the message to the client.
			response.setContentType("text/plain");
			response.setContentLength(message.length());
			PrintWriter out = response.getWriter();
			out.println(message+"\r\n");

		}


	}


	/*
	 * This method should perform a lookup of the given SA and verify the claimed authenticity
	 */
	private boolean lookupUser(String SA) {
		  if(SA != null) {
			  return true;
		 } else {
			 return false;
		 }
	}

	/*
	 * Treat POST requests similar as GET requests
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
			doGet(request, response);
	}
}