package com;

import java.io.*;
import javax.microedition.io.*;
import core.ServiceSupplicant;
import core.Controller;

/**
 * The ClientProxy class is a thread class running in the background.
 * It creates the ServerSocketConnection which the client is connecting to!
 * @author Hvard Holje
 * @since May 4, 2007
 *
 */
public class ClientProxy extends Thread {

	private ServiceSupplicant midlet;
	private int type;
	private Controller controller;

	private String socketAddr = "socket://:4035";
	private ServerSocketConnection serverCon;
	private SocketConnection con;
	private OutputStream os;
	private InputStream is;
	private Sender sender;

	public ClientProxy(ServiceSupplicant midlet, Controller controller, int type) {
		this.midlet = midlet;
		this.controller = controller;
		this.type = type;
	}


	/**
	 * All I/O handling is executing in a Thread
	 */
	public synchronized void run() {

		try {
			connect();

			is = con.openInputStream();
			os = con.openOutputStream();
			sender = new Sender(os); //This handles the output stream

			while(true) { //This will loop until the ServerConnection is closed

				StringBuffer sb = new StringBuffer();
                int c = 0;

                while (((c = is.read()) != '\n') && (c != -1)) {
                    sb.append((char)c);
                }
                midlet.print(sb.toString());
                controller.handleInputStream(sb.toString().trim(), type);

                if (c == -1) {
                    break;
                }
			}
		}

		catch(IOException ioe) {
			System.out.println(ioe);
		}

		finally {
            try {
                if (is != null) {
                    is.close();
                    System.out.print("SupplicantProxy -> InputStream closed   ");
                }
                if (serverCon != null) {
                	serverCon.close();
                    System.out.print("SupplicantProxy -> ServerSocketConnection closed   ");
                }
                if (con != null) {
                	con.close();
                    System.out.print("SupplicantProxy -> SocketConnection closed   ");
                }
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }

	} //END run()


	/**
	 * Setting up the ServerSocketConnection
	 * This should last during the hole user session!
	 */
	public void connect() {

		try {
			serverCon = (ServerSocketConnection)Connector.open(socketAddr);

			//Debug
			midlet.print("Waiting for client to connect..");
			con = (SocketConnection) serverCon.acceptAndOpen();
			midlet.print("Connected to client!");
		}

		catch (ConnectionNotFoundException ce) {
	        System.out.println("Connection to server not found...." + ce.toString());
		}

		catch(IOException ioe){
			System.out.println("SupplicantProxy: " + ioe.toString());
		}


	} //END connect()


	public void sendResponse(String response) {
		sender.send(response);
	}

	/**
	 * Helper class for sending messages to client
	 * Copyright  2007 Sun Microsystems, Inc. All rights reserved.
	 * This code is borrowed from "NetworkDemo" in Sun Wireless Toolkit 2.5
	 */
	class Sender extends Thread {
	    private OutputStream os;
	    private String message;

	    public Sender(OutputStream os) {
	        this.os = os;
	        start();
	    }

	    public synchronized void send(String msg) {
	        message = msg;
	        notify();
	    }

	    public synchronized void run() {
	        while (true) {
	            // If no client to deal, wait until one connects
	            if (message == null) {
	                try {
	                    wait();
	                } catch (InterruptedException e) {
	                }
	            }

	            if (message == null) {
	                break;
	            }

	            try {
	                os.write(message.getBytes());
	                os.write("\r\n".getBytes());
	            } catch (IOException ioe) {
	                ioe.printStackTrace();
	            }

	            // Completed client handling, return handler to pool and
	            // mark for wait
	            message = null;
	        }
	    }

	    public synchronized void stop() {
	        message = null;
	        notify();
	    }
	}


} //END class SupplicantProxy
