package core;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.midlet.MIDlet;

import com.ClientProxy;

/**
 * The ServiceSupplicant class is the core of the Supplicant and is 
 * responsible for all communication between the components. 
 * 
 * It creates a ServerSocketConnection listening to port xx
 * and reveives incoming requests from the clients. 
 * It interprets the requests and contacts the SIMSupplicant when
 * it needs user credentials from the SIM. 
 * 
 * It is also supposed to implement the EAP-SIM framework, 
 * to be able to communicate with the Identity Provider. 
 * 
 * @author Hvard Holje
 * @since May 4, 2007
 */
public class ServiceSupplicant extends MIDlet implements CommandListener {
    
	private ClientProxy proxy;
	
	private Controller controller;
	
	private Command exitCommand;

    private Display display = null;

    private Form frmMain;
    
    private final int VERIFY_PIN = 1;
    
    public ServiceSupplicant() {
        exitCommand = new Command("Exit", Command.EXIT, 1);
        frmMain = new Form("Status:");
        frmMain.addCommand(exitCommand);
        frmMain.setCommandListener(this);
    }

    protected void startApp() {
    	if (display == null) {
            display = Display.getDisplay(this);
        }
    	
        display.setCurrent(frmMain);
        init();
    }

    protected void pauseApp() {}
    
    protected void destroyApp(boolean bool) {}

    public void commandAction(Command cmd, Displayable disp) {
        if (cmd == exitCommand) {
            destroyApp(false);
            notifyDestroyed();
        } 
    }
    
    /**
     * This method initializes the Socket connection listening to incoming requests
     */
    private void init() {
    	//Initializes the Controller object
    	controller = new Controller(this);

    	proxy = new ClientProxy(this, controller,VERIFY_PIN);
    	proxy.start();
    }
    
    public void sendResponseToClientApp(int type, String response) {
    	proxy.sendResponse(response);
    }
    
    /**
     * DEBUG
     * @param value
     */
    public void print(String value) {
    	frmMain.append("\n" + value);
    }

}
