package core;

import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.*;

import java.io.*;
import base.AuthenticationRequest;
import base.AuthenticationResponse;

/**
 * The XmlParser class is parsing objects to and from XML. 
 * Utilizing the KXML-parser, which is a lightweight xml parser developed for
 * use on mobile handsets. 
 */
public class XmlParser {
    private String root;
    private String rootChild;
    AuthenticationRequest authReq;
    
    public XmlParser() {}
    
    /**
     * This method creates a AuthenticationRequest-object from the
     * xml data parsed from the inputStream
     * @param is
     * @return AuthenticationRequest
     */
    public AuthenticationRequest getAuthenticationRequest(String input) {
    	root = "AuthenticationRequest";    
    	rootChild = "Pin";
        
    	KXmlParser parser = new KXmlParser();
        
    	try {
    		
    		//Converts input string to stream data
    		byte[] xmlByteArray = input.getBytes();
			ByteArrayInputStream xmlStream = new 
			ByteArrayInputStream( xmlByteArray );
			InputStreamReader xmlReader = new InputStreamReader( xmlStream );
			parser.setInput(xmlReader);
			//parser.setInput(new InputStreamReader(is));

			parser.nextTag();
	    	parser.require(XmlPullParser.START_TAG, null, root);
	    	
	    	while(parser.nextTag() == XmlPullParser.START_TAG) {
	    	  parser.require(XmlPullParser.START_TAG, null, rootChild);

	    	   // handle element content
	    	  String pin = parser.nextText();
	    	  System.out.println("PIN: "+ pin);
	    	  authReq = new AuthenticationRequest(pin);
	    	  parser.require(XmlPullParser.END_TAG, null, "Pin");
	    	}

	    	parser.require(XmlPullParser.END_TAG, null, "AuthenticationRequest");
	
            
         } catch (Exception e) {
             e.printStackTrace ();
             System.out.println("Error in getAuthenticationRequest(): " + e);
             return null;
         }
         return authReq;
    }
    
    /**
     * Creates the xml AuthenticationResponse that proves the client is authenticated
     * @param authResponse
     * @return String xmlAuthResponse
     */
    public String createAuthenticationResponse(AuthenticationResponse authResponse) {
    	String SA = authResponse.getSecurityAssociation(); 
    	String xml = //"<?xml version=\"1.0\" encoding=\"ISO-8859-1\">" +
    			"<AuthenticationResponse>" +
    			"<SecurityAssociation>"+SA+"</SecurityAssociation>" +
    			"<ErrorDescription></ErrorDescription>" +
    			"</AuthenticationResponse>";
    	
    	return xml.trim();
    }

    /**
     * This method reads the inputStream and parses the xml data
     * @param parser KXmlParser
     * @param flag int
     * @throws IOException
     * @throws XmlPullParserException
     * @throws NumberFormatException
     */
    private void parseXml(KXmlParser parser, int flag) throws IOException, XmlPullParserException, NumberFormatException {

        parser.require(XmlPullParser.START_TAG, null, rootChild);
        String pin = "";
        while (parser.nextTag() != XmlPullParser.END_TAG) {

            parser.require(XmlPullParser.START_TAG, null,null);
            String name = parser.getName();
            String text = parser.nextText();
            System.out.println ("<"+name+">"+text);

            switch (flag) {

            case 1: //PIN
            	parser.require(XmlPullParser.START_TAG, null,name);
            	//Debug
            	System.out.print("*** parser.getName():" + name + "***");
               
                //Gets the attribute(s) - in alphabetic order
                pin = parser.getAttributeValue(0);
                String p = parser.nextText(); //Parse error uten denne?
                authReq = new AuthenticationRequest(pin);
                break;
           
            default:
            	System.out.print("*** Break***");
                break;
            } //switch
            
            parser.require(XmlPullParser.END_TAG, null, name);
        
        } //while
        
        parser.require(XmlPullParser.END_TAG, null, rootChild); //Adds the rootChild end-tag

   }

}